#!/usr/bin/env python
#
# Copyright 2020 Flavio Goncalves Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from . import MINIMUM_KEY_SIZE
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.asymmetric.ec import (
    EllipticCurvePrivateKey, EllipticCurvePublicKey
)
from cryptography.hazmat.primitives.asymmetric.rsa import (
    RSAPrivateKey, RSAPublicKey
)
from cryptography.hazmat.primitives.serialization import (
    load_pem_private_key,
    load_pem_public_key,
    Encoding,
    PrivateFormat,
    PublicFormat,
    NoEncryption,
)

import logging

logger = logging.getLogger(__name__)


def generate_key(**kwargs):
    size = kwargs.get("size", MINIMUM_KEY_SIZE)
    curve = kwargs.get("curve")
    _type = kwargs.get('type', "rsa")

    if _type.lower() == "rsa":
        from .rsa import generate_key
        return generate_key(size)
    elif _type.lower() == "ec":
        if curve:
            from .ec import generate_key
            return generate_key(curve)
        else:
            logger.warning("Please inform a curve in order to generate an "
                           "Elliptical Curve key.")
            return None
    raise NotImplementedError


def pem_to_private_key(data):
    """
    Load a PEM-encoded private key.
    """
    key = load_pem_private_key(data, password=None, backend=default_backend())
    if not isinstance(key, (RSAPrivateKey, EllipticCurvePrivateKey)):
        raise NotImplementedError("Key is not a private RSA or EC key.")
    elif isinstance(key, RSAPrivateKey) and key.key_size < MINIMUM_KEY_SIZE:
        raise ValueError("The key must be %s bits or longer." %
                         MINIMUM_KEY_SIZE)
    return key


def pem_to_public_key(data):
    """ Load a PEM-encoded public key.

    :param data:
    :return:
    """
    key = load_pem_public_key(data, backend=default_backend())
    if not isinstance(key, (RSAPublicKey, EllipticCurvePublicKey)):
        raise NotImplementedError("Key is not a public RSA or EC key.")
    elif isinstance(key, RSAPublicKey) and key.key_size < MINIMUM_KEY_SIZE:
        raise ValueError("The key must be %s bits or longer." %
                         MINIMUM_KEY_SIZE)
    return key


def key_to_pem(key):
    """
    Export a private key to PEM format.
    """
    if hasattr(key, "private_bytes"):
        return key.private_bytes(Encoding.PEM,
                                 PrivateFormat.TraditionalOpenSSL,
                                 NoEncryption())
    elif hasattr(key, "public_bytes"):
        if isinstance(key, RSAPublicKey):
            return key.public_bytes(Encoding.PEM,
                                    PublicFormat.PKCS1)
        return key.public_bytes(Encoding.PEM,
                                PublicFormat.SubjectPublicKeyInfo)



# -*- coding: UTF-8 -*-
#
# Copyright 2020 Flavio Goncalves Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

logger = logging.getLogger(__name__)


class PeasantTransport(object):

    _peasant: 'Peasant'

    def __init__(self):
        self._peasant = None

    @property
    def peasant(self):
        return self._peasant

    @peasant.setter
    def peasant(self, peasant):
        self._peasant = peasant

    def get(self, path, **kwargs):
        raise NotImplementedError

    def head(self, path, **kwargs):
        raise NotImplementedError

    def post(self, path, **kwargs):
        raise NotImplementedError

    def post_as_get(self, path, **kwargs):
        raise NotImplementedError

    def set_directory(self):
        raise NotImplementedError

    def new_nonce(self):
        raise NotImplementedError

    def is_registered(self):
        raise NotImplementedError


class Peasant(object):

    _transport: PeasantTransport

    def __init__(self, transport):
        self._directory_cache = None
        self._transport = transport
        self._transport.peasant = self

    @property
    def directory_cache(self):
        return self._directory_cache

    @directory_cache.setter
    def directory_cache(self, directory_cache):
        self._directory_cache = directory_cache

    @property
    def transport(self):
        return self._transport

    def directory(self):
        if self.directory_cache is None:
            self.transport.set_directory()
        return self.directory_cache

    def new_nonce(self):
        return self.transport.new_nonce()


class AsyncPeasant(Peasant):

    def __init__(self, transport):
        super(AsyncPeasant, self).__init__(transport)

    async def directory(self):
        if self._directory_cache is None:
            future = self.transport.set_directory()
            if future is not None:
                logger.debug("Running transport set directory cache "
                             "asynchronously.")
                await future
        return self._directory_cache



#!/usr/bin/env python
#
# Copyright 2020 Flavio Goncalves Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from . import MINIMUM_KEY_SIZE

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.asymmetric.rsa import (
    generate_private_key,
)

import logging

logger = logging.getLogger(__name__)


def generate_key(size=MINIMUM_KEY_SIZE):
    """
    Generates a new RSA private key.
    """
    return generate_private_key(65537, size, default_backend())



# -*- coding: UTF-8 -*-
#
# Copyright 2020 Flavio Goncalves Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import functools
import logging

logger = logging.getLogger(__name__)


class NonceServiceMixin:

    def consume(self, **kwargs):
        """ Handle a nonce sent to the request
        :param kwargs:
        :key request: The Http request being serviced.
        :key nonce: The nonce being consumed by the server.
        :return :
        """
        raise NotImplementedError

    def clear(self, **kwargs):
        """ Clears a nonce sent to the request
        :param kwargs:
        :key request: The Http request being serviced.
        :key nonce: The nonce being consumed by the server.
        """
        raise NotImplementedError

    def block_request(self, **kwargs):
        raise NotImplementedError

    def from_request(self, **kwargs):
        raise NotImplementedError

    def provided(self, **kwargs):
        raise NotImplementedError


def nonced(method):
    """ Decorates a handler to only accept requests with nonce header.
    If the request is missing the request handler we set the status as 400 with
    a malformed message.
    """
    @functools.wraps(method)
    def wrapper(self, *args, **kwargs):
        if self.nonce_service.provided(request=self):
            nonce = self.nonce_service.from_request(request=self)
            if self.nonce_service.consume(
                    request=self, nonce=nonce) is not None:
                retval = method(self, *args, **kwargs)
                self.nonce_service.clear(request=self, nonce=nonce)
                return retval
        else:
            self.nonce_service.block_request(request=self)
    return wrapper
